# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import NetAppManagementClientConfiguration
from .operations import (
    AccountsOperations,
    BackupPoliciesOperations,
    BackupVaultsOperations,
    BackupsOperations,
    BackupsUnderAccountOperations,
    BackupsUnderBackupVaultOperations,
    BackupsUnderVolumeOperations,
    NetAppResourceOperations,
    NetAppResourceQuotaLimitsOperations,
    NetAppResourceRegionInfosOperations,
    Operations,
    PoolsOperations,
    SnapshotPoliciesOperations,
    SnapshotsOperations,
    SubvolumesOperations,
    VolumeGroupsOperations,
    VolumeQuotaRulesOperations,
    VolumesOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class NetAppManagementClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Microsoft NetApp Files Azure Resource Provider specification.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.netapp.aio.operations.Operations
    :ivar net_app_resource: NetAppResourceOperations operations
    :vartype net_app_resource: azure.mgmt.netapp.aio.operations.NetAppResourceOperations
    :ivar net_app_resource_quota_limits: NetAppResourceQuotaLimitsOperations operations
    :vartype net_app_resource_quota_limits:
     azure.mgmt.netapp.aio.operations.NetAppResourceQuotaLimitsOperations
    :ivar net_app_resource_region_infos: NetAppResourceRegionInfosOperations operations
    :vartype net_app_resource_region_infos:
     azure.mgmt.netapp.aio.operations.NetAppResourceRegionInfosOperations
    :ivar accounts: AccountsOperations operations
    :vartype accounts: azure.mgmt.netapp.aio.operations.AccountsOperations
    :ivar pools: PoolsOperations operations
    :vartype pools: azure.mgmt.netapp.aio.operations.PoolsOperations
    :ivar volumes: VolumesOperations operations
    :vartype volumes: azure.mgmt.netapp.aio.operations.VolumesOperations
    :ivar snapshots: SnapshotsOperations operations
    :vartype snapshots: azure.mgmt.netapp.aio.operations.SnapshotsOperations
    :ivar snapshot_policies: SnapshotPoliciesOperations operations
    :vartype snapshot_policies: azure.mgmt.netapp.aio.operations.SnapshotPoliciesOperations
    :ivar backup_policies: BackupPoliciesOperations operations
    :vartype backup_policies: azure.mgmt.netapp.aio.operations.BackupPoliciesOperations
    :ivar volume_quota_rules: VolumeQuotaRulesOperations operations
    :vartype volume_quota_rules: azure.mgmt.netapp.aio.operations.VolumeQuotaRulesOperations
    :ivar volume_groups: VolumeGroupsOperations operations
    :vartype volume_groups: azure.mgmt.netapp.aio.operations.VolumeGroupsOperations
    :ivar subvolumes: SubvolumesOperations operations
    :vartype subvolumes: azure.mgmt.netapp.aio.operations.SubvolumesOperations
    :ivar backups: BackupsOperations operations
    :vartype backups: azure.mgmt.netapp.aio.operations.BackupsOperations
    :ivar backup_vaults: BackupVaultsOperations operations
    :vartype backup_vaults: azure.mgmt.netapp.aio.operations.BackupVaultsOperations
    :ivar backups_under_backup_vault: BackupsUnderBackupVaultOperations operations
    :vartype backups_under_backup_vault:
     azure.mgmt.netapp.aio.operations.BackupsUnderBackupVaultOperations
    :ivar backups_under_volume: BackupsUnderVolumeOperations operations
    :vartype backups_under_volume: azure.mgmt.netapp.aio.operations.BackupsUnderVolumeOperations
    :ivar backups_under_account: BackupsUnderAccountOperations operations
    :vartype backups_under_account: azure.mgmt.netapp.aio.operations.BackupsUnderAccountOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2024-03-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = NetAppManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.net_app_resource = NetAppResourceOperations(self._client, self._config, self._serialize, self._deserialize)
        self.net_app_resource_quota_limits = NetAppResourceQuotaLimitsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.net_app_resource_region_infos = NetAppResourceRegionInfosOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.accounts = AccountsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.pools = PoolsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.volumes = VolumesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.snapshots = SnapshotsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.snapshot_policies = SnapshotPoliciesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_policies = BackupPoliciesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.volume_quota_rules = VolumeQuotaRulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.volume_groups = VolumeGroupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.subvolumes = SubvolumesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups = BackupsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backup_vaults = BackupVaultsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backups_under_backup_vault = BackupsUnderBackupVaultOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backups_under_volume = BackupsUnderVolumeOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backups_under_account = BackupsUnderAccountOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
